@@ -7,9 +7,13 @@ import os
 import re
 import tempfile
 import shutil
+import urllib
+import glob
 from optparse import OptionParser
 
-from dbtexmf.core.confparser import DbtexConfig, texinputs_parse
+from dbtexmf.core.confparser import DbtexConfig, texinputs_parse, texstyle_parse
+from dbtexmf.xslt import xslt
+from dbtexmf.core.logger import logger
 
 
 def suffix_replace(path, oldext, newext=""):
@@ -20,10 +24,10 @@ def suffix_replace(path, oldext, newext=""):
         return (path+newext)
 
 def path_to_uri(path):
-    if os.sep != "/":
-        return path.replace(os.sep, "/")
+    if os.name == 'nt':
+        return 'file:///' + urllib.pathname2url(path).replace('|', ':', 1)
     else:
-        return path
+        return urllib.pathname2url(path)
 
 
 class DbTex:
@@ -46,8 +50,11 @@ class DbTex:
         self.tmpdir = ""
         self.tmpdir_user = None
         self.fig_paths = []
+        self.bib_paths = []
+        self.bst_paths = []
         self.texinputs = []
         self.texbatch = 1
+        self.texpost = ""
         self.fig_format = ""
         self.backend = ""
 
@@ -68,6 +75,7 @@ class DbTex:
         self.xslmain = os.path.join(self.topdir, "xsl", "docbook.xsl")
         self.xsllist = os.path.join(self.topdir, "xsl", "common", "mklistings.xsl")
         self.texdir = os.path.join(self.topdir, "texstyle")
+        self.texlocal = ""
         self.confdir = os.path.join(self.topdir, "confstyle")
 
     def update_texinputs(self):
@@ -79,20 +87,26 @@ class DbTex:
             texinputs = sep + "%s%s" % (texpaths, texinputs)
         else:
             texinputs = sep + "%s%s%s" % (texpaths, sep, texinputs)
+        # Texlocal has precedence over the standard (and added) paths
+        if self.texlocal:
+            texinputs = self.texlocal + "//" + sep + texinputs
         os.environ["TEXINPUTS"] = texinputs
 
+    def set_xslt(self, xsltmod=None):
+        # Set the XSLT to use. Set a default XSLT if none specified.
+        # One can replace an already defined XSLT if explicitely required.
+        if not(xsltmod):
+            if self.xsltproc:
+                return
+            xsltmod = "xsltproc"
+        self.xsltproc = xslt.load(xsltmod)
+
     def set_format(self, format):
         if not(format in ("rtex", "tex", "dvi", "ps", "pdf")):
             raise ValueError("unknown format '%s'" % format)
         else:
             self.format = format
 
-    def set_verbose(self, verbose):
-        self.verbose = verbose
-        self.runtex.verbose = verbose
-        self.rawtex.verbose = verbose
-        self.xsltproc.verbose = verbose
-
     def unset_flags(self, what):
         self.flags &= ~what
 
@@ -132,7 +146,7 @@ class DbTex:
         self.xslbuild = wrapper
 
     def make_xml(self):
-        print "Build the XML file..."
+        self.log.info("Build the XML file...")
         xmlfile = self.basefile + ".xml"
         self.sgmlxml.run(self.input, xmlfile)
         self.input = xmlfile
@@ -140,44 +154,66 @@ class DbTex:
     def make_listings(self):
         self.listings = os.path.join(self.tmpdir, "listings.xml")
         if (self.flags & self.USE_MKLISTINGS):
-            print "Build the listings..."
-            param = ("current.dir", self.inputdir)
+            self.log.info("Build the listings...")
+            param = {"current.dir": self.inputdir}
             self.xsltproc.use_catalogs = 0
             self.xsltproc.run(self.xsllist, self.input,
-                              self.listings, params=[param])
+                              self.listings, params=param)
         else:
-            print "No external file support"
+            self.log.info("No external file support")
             f = file(self.listings, "w")
             f.write("<listings/>\n")
             f.close()
 
     def make_rawtex(self):
         self.rawfile = self.basefile + ".rtex"
-        param = ("listings.xml", self.listings)
+        param = {"listings.xml": self.listings,
+                 "current.dir": self.inputdir}
         self.xsltproc.use_catalogs = 1
         self.xsltproc.run(self.xslbuild, self.input,
-                          self.rawfile, opts=self.xslopts, params=[param])
+                          self.rawfile, opts=self.xslopts, params=param)
 
     def make_tex(self):
         self.texfile = self.basefile + ".tex"
-        self.rawtex.set_format(self.format)
+        self.rawtex.set_format(self.format, self.backend)
         if self.fig_format:
             self.rawtex.fig_format(self.fig_format)
 
         # By default figures are relative to the source file directory
         self.rawtex.set_fig_paths([self.inputdir] + self.fig_paths)
-        self.rawtex.parse(self.rawfile, self.texfile)
+
+        self.rawfiles = glob.glob("*.rtex")
+        for rawfile in self.rawfiles:
+            texfile = os.path.splitext(rawfile)[0] + ".tex"
+            self.rawtex.parse(rawfile, texfile)
 
     def make_bin(self):
         self.binfile = self.basefile + "." + self.format
         if self.backend:
             self.runtex.set_backend(self.backend)
+        self.runtex.texpost = self.texpost
         self.runtex.set_fig_paths([self.inputdir] + self.fig_paths)
+        self.runtex.set_bib_paths([self.inputdir] + self.bib_paths,
+                                  [self.inputdir] + self.bst_paths)
+
+        # Build the dependent files
+        self.rawfiles.remove(self.rawfile)
+        for rawfile in self.rawfiles:
+            texfile = os.path.splitext(rawfile)[0] + ".tex"
+            binfile = os.path.splitext(rawfile)[0] + "." + self.format
+            self.log.info("Build %s" % binfile)
+            self.runtex.compile(texfile, binfile, self.format,
+                                batch=self.texbatch)
+            self.runtex.clean()
+
+        # Build the main document file 
+        self.log.info("Build %s" % self.binfile)
         self.runtex.compile(self.texfile, self.binfile, self.format,
                             batch=self.texbatch)
         self.runtex.clean()
 
     def compile(self):
+        self.set_xslt()
         self.cwdir = os.getcwd()
         self.tmpdir = self.tmpdir_user or tempfile.mkdtemp()
         self.inputdir = os.path.dirname(self.input)
@@ -231,8 +267,11 @@ class DbTex:
         return self.binfile
 
 
+dump_stack = False
 def failed_exit(msg, rc=1):
+    global dump_stack
     print >>sys.stderr, (msg)
+    if dump_stack: raise
     sys.exit(rc)
 
 
@@ -254,6 +293,8 @@ class DbTexCommand:
         parser.add_option("-d", "--debug", action="store_true",
                           help="Debug mode. Keep the temporary directory in "
                                "which %s actually works" % prog)
+        parser.add_option("-D", "--dump", action="store_true",
+                          help="Dump error stack (debug purpose)")
         parser.add_option("-f", "--fig-format",
                           help="Input figure format, used when not deduced from "
                                "figure extension")
@@ -264,6 +305,14 @@ class DbTexCommand:
         parser.add_option("-I", "--fig-path", action="append",
                           dest="fig_paths", metavar="FIG_PATH",
                           help="Additional lookup path of the figures")
+        parser.add_option("-l", "--bst-path", action="append",
+                          dest="bst_paths", metavar="BST_PATH",
+                          help="Bibliography style file path")
+        parser.add_option("-L", "--bib-path", action="append",
+                          dest="bib_paths", metavar="BIB_PATH",
+                          help="BibTeX database path")
+        parser.add_option("-m", "--xslt",
+                          help="XSLT engine to use. (default=xsltproc)")
         parser.add_option("-o", "--output", dest="output",
                           help="Output filename. When not used, the input filename "
                                "is used, with the suffix of the output format")
@@ -272,6 +321,15 @@ class DbTexCommand:
         parser.add_option("-P", "--param", dest="xslparams",
                           action="append", metavar="PARAM=VALUE",
                           help="Set an XSL parameter value from command line")
+        parser.add_option("-r", "--texpost", metavar="SCRIPT",
+                          help="Script called at the very end of the tex "
+                               "compilation. Its role is to modify the tex file "
+                               "or one of the compilation file before the last "
+                               "round.")
+        parser.add_option("-s", "--texstyle", metavar="STYFILE",
+                          help="Latex style to apply. It can be a package name, or "
+                               "directly a package path that must ends with "
+                               "'.sty'")
         parser.add_option("-t", "--type", dest="format",
                           help="Output format. Available formats:\n"
                                "tex, dvi, ps, pdf (default=pdf)")
@@ -289,7 +347,7 @@ class DbTexCommand:
                           help="Print the %s version" % prog)
         parser.add_option("-V", "--verbose", action="store_true",
                           help="Verbose mode, showing the running commands")
-        parser.add_option("-x", "--xslt", dest="xslopts",
+        parser.add_option("-x", "--xslt-opts", dest="xslopts",
                           action="append", metavar="XSLT_OPTIONS",
                           help="Arguments directly passed to the XSLT engine")
         parser.add_option("-X", "--no-external", action="store_true",
@@ -319,11 +377,17 @@ class DbTexCommand:
             except Exception, e:
                 failed_exit("Error: %s" % e)
 
+        # Always set the XSLT (default or not)
+        try:
+            run.set_xslt(options.xslt)
+        except Exception, e:
+            failed_exit("Error: %s" % e)
+
         if options.xslopts:
             run.xslopts = options.xslopts
 
         if options.xslparams:
-            run.xslparams = options.xslparams
+            run.xslparams += options.xslparams
 
         if options.debug:
             run.debug = options.debug
@@ -331,6 +395,20 @@ class DbTexCommand:
         if options.fig_paths:
             run.fig_paths += [os.path.realpath(p) for p in options.fig_paths]
 
+        if options.bib_paths:
+            run.bib_paths += [os.path.realpath(p) for p in options.bib_paths]
+
+        if options.bst_paths:
+            run.bst_paths += [os.path.realpath(p) for p in options.bst_paths]
+
+        if options.texstyle:
+            try:
+                xslparam, texpath = texstyle_parse(options.texstyle)
+            except Exception, e:
+                failed_exit("Error: %s" % e)
+            run.xslparams.append(xslparam)
+            if texpath: run.texinputs.append(texpath)
+
         if options.texinputs:
             for texinputs in options.texinputs:
                 run.texinputs += texinputs_parse(texinputs)
@@ -353,11 +431,17 @@ class DbTexCommand:
                 failed_exit("Error: '%s' does not exist" % options.xsl_user)
             run.xsluser = xsluser
 
+        if options.texpost:
+            path = os.path.realpath(options.texpost)
+            if not(os.path.isfile(path)):
+                failed_exit("Error: '%s' does not exist" % options.texpost)
+            run.texpost = path
+
         if options.no_external:
             run.unset_flags(run.USE_MKLISTINGS)
 
         if options.verbose:
-            run.set_verbose(options.verbose)
+            run.verbose = options.verbose
     
         if options.tmpdir:
             if not(os.path.exists(options.tmpdir)):
@@ -365,7 +449,31 @@ class DbTexCommand:
                     os.mkdir(options.tmpdir)
                 except Exception, e:
                     failed_exit("Error: %s" % e)
-            run.tmpdir_user = options.tmpdir
+            run.tmpdir_user = os.path.abspath(options.tmpdir)
+
+        if options.dump:
+            global dump_stack
+            dump_stack = True
+
+    def get_config_paths(self):
+        # Allows user directories where to look for configuration files
+        paths = [os.getcwd()]
+        paths.append(os.path.expanduser(os.path.join("~", "."+self.prog)))
+
+        # Unix specific system-wide config files
+        if "posix" in sys.builtin_module_names:
+            paths.append(os.path.join("/etc", self.prog))
+
+        # Last but not least, the tool config dir
+        paths.append(self.run.confdir)
+
+        # Optionally the paths from an environment variable
+        conf_paths = os.getenv("DBLATEX_CONFIG_FILES")
+        if not(conf_paths):
+            return paths
+
+        paths += conf_paths.split(os.pathsep)
+        return paths
 
     def main(self):
         (options, args) = self.parser.parse_args()
@@ -387,7 +495,8 @@ class DbTexCommand:
         conf = DbtexConfig()
         if options.style:
             try:
-                conf.fromstyle(options.style, [run.confdir])
+                conf.paths = self.get_config_paths()
+                conf.fromstyle(options.style)
             except Exception, e:
                 failed_exit("Error: %s" % e)
             
@@ -404,6 +513,9 @@ class DbTexCommand:
         # Now apply the command line setup
         self.run_setup(options)
 
+        # Verbose mode
+        run.log = logger(self.prog, run.verbose)
+
         input = os.path.realpath(args[0])
 
         # The output name can be deduced from the input one:
